package main

import (
	"encoding/json"
	"github.com/stretchr/testify/assert"
	"golang.org/x/net/context"
	"google.golang.org/grpc"
	"git.loafle.net/overflow/crawler_go"
	pb "git.loafle.net/overflow/crawler_go/grpc"
	"log"
	"testing"
	"time"
	"git.loafle.net/overflow/crawler_go/health_crawler/redis_protocol_crawler_go"
	config "git.loafle.net/overflow/agent_api/config_manager"
	"git.loafle.net/overflow/rpc_network_crawler_go/impl"
)

type call func() crawler.Crawler

func startRPCServerTest(cc call) {
	initCrawlers()
	rr := cc()
	AddDelegate(pb.Crawlers_HEALTH_REDIS.String(), rr)
	go start(&impl.ConfigServerImpl{Crawlers : g_crawlers}, &impl.DataServerImpl{Crawlers : g_crawlers},&impl.StatusServerImpl{})
}

func clientCall(t *testing.T, cl pb.Crawlers) {
	time.Sleep(2 * time.Second)
	conn, err := grpc.Dial("localhost:50000", grpc.WithInsecure())
	if err != nil {
		log.Fatalf("did not connect: %v", err)
	}
	defer conn.Close()
	c := pb.NewDataClient(conn)

	in := &pb.Input{
		Name: cl,
		Id:   "test_redis_sid",
	}

	out, err := c.Get(context.Background(), in)
	if err != nil {
		log.Fatalf("could not greet: %v", err)
	}

	var check bool
	json.Unmarshal(out.Data, &check)
	assert.Equal(t, true, check)
}

func TestRedisGRPC(t *testing.T) {



	go startRPCServerTest(func() crawler.Crawler {
		rr := redis_protocol_crawler_go.NewRedisHeahthCrawler()
		//m := make(map[string]interface{}, 0)
		//m["ip"] = "192.168.1.104"
		//m["port"] = "6379"
		//m["portType"] = "tcp"
		//m["ssl"] = false
		
		m := config.Config{}
		m.Target = config.Target{}
		m.Target.Connection = config.Connection{}
		m.Target.Connection.Ip = "192.168.1.215"
		m.Target.Connection.Port = "6379"
		m.Target.Connection.PortType = "tcp"
		m.Target.Connection.SSL = false
		rr.PutConfig("test_redis_sid", m)

		return rr
	})
	clientCall(t, pb.Crawlers_HEALTH_REDIS)
}

func TestStatusCall(t *testing.T) {
	go startRPCServerTest(func()crawler.Crawler {return nil })
	
	time.Sleep(2 * time.Second)
	conn, err := grpc.Dial("localhost:50000", grpc.WithInsecure())
	if err != nil {
		log.Fatalf("did not connect: %v", err)
	}
	defer conn.Close()
	c := pb.NewStatusClient(conn)
	
	in := &pb.Empty{}
	
	out, err := c.Status(context.Background(), in)
	if err != nil {
		log.Fatalf("could not greet: %v", err)
	}
	
	assert.Equal(t, true, out.Check)
	
}



//type configcall func()
//
//////////////////////////////////////// redis rpc test start -->
//func TestRedis(t *testing.T) {
//	go initTestServer(func() {
//		c := of_rpc.GetCrawler(crawler.REDIS.String()).(*redis_protocol_crawler_go.RedisHeahthCrawler)
//		m := make(map[string]interface{}, 0)
//		m["ip"] = "192.168.1.104"
//		m["port"] = "6379"
//		m["portType"] = "tcp"
//		m["ssl"] = false
//		c.PutConfig("test_redis_sid", m)
//	})
//	time.Sleep(2 * time.Second)
//	in := param.Input{
//		Name: crawler.REDIS.String(),
//		Id:   "test_redis_sid",
//	}
//	checkhealth(t, in)
//}
//////////////////////////////////////// redis rpc test end <--
//
//////////////////////////////////////// imap rpc test start -->
//func TestImap(t *testing.T) {
//	go initTestServer(func() {
//		c := of_rpc.GetCrawler(crawler.IMAP.String()).(*imap_protocol_crawler_go.IMAPHealthCrawler)
//		m := make(map[string]interface{}, 0)
//		m["ip"] = "192.168.1.215"
//		m["port"] = "993"
//		m["portType"] = "tcp"
//		m["ssl"] = true
//		c.PutConfig("test_imap_sid", m)
//	})
//	time.Sleep(2 * time.Second)
//	in := param.Input{
//		Name: crawler.IMAP.String(),
//		Id:   "test_imap_sid",
//	}
//	checkhealth(t, in)
//}
//////////////////////////////////////// imap rpc test end <--
//
//////////////////////////////////////// ftp rpc test start -->
//func TestFTP(t *testing.T) {
//	go initTestServer(func() {
//		c := of_rpc.GetCrawler(crawler.FTPS.String()).(*ftps_protocol_crawler_go.FTPSHealthCrawler)
//		m := make(map[string]interface{}, 0)
//		m["ip"] = "192.168.1.103"
//		m["port"] = "2121"
//		m["portType"] = "tcp"
//		m["ssl"] = false
//		c.PutConfig("test_ftps_sid", m)
//	})
//	time.Sleep(2 * time.Second)
//	in := param.Input{
//		Name: crawler.FTPS.String(),
//		Id:   "test_ftps_sid",
//	}
//	checkhealth(t, in)
//}
//////////////////////////////////////// ftp rpc test end <--
//
//
//
//func initTestServer(config configcall) {
//	initCrawlers()
//	config()
//	start()
//}
//
//func checkhealth(t *testing.T, in param.Input) {
//	result, err := client.InvokeJSONRPCGet("50000", in)
//
//	if err != nil {
//		t.Fatal(err)
//	}
//
//	var check bool
//	json.Unmarshal(result.Data, &check)
//	assert.Equal(t, true, check)
//}
